#!/bin/bash

echo === Zytronic Touchscreen Driver Install Script =================

# This file is run as a part of the generalized installer
# for the Zytronic Touchscreen Linux Drivers

# It should move the source files to the /usr/src/module/zytronic directory
# and make, make install, modprobe and depmod

# Also, it should manage the rebinding scripts and calibration activities if
# required through the init system.

# When run successfully on the target machine, the machine should be 
# fully configured to use the ZXY... touch screen controller.

haltFile=../halted

#####i Clear any previous HALT flag
rm -f $haltFile
KERN_RELEASE=`uname -r`
echo Running linux kernel: $KERN_RELEASE

##### Check for existing driver
TEXT=`./find-elements.sh`
if [ $? -ne 0 ]; then
	echo -e "\n\nWarning: There seems to be a Zytronic Driver already installed!"
	echo $TEXT
	echo -e "\n\n"
	echo "Please remove it. (see uninstall.sh)"
	touch $haltFile
	# do not stop yet
fi

##### Check for make & gcc
which make > /dev/null
if [ $? -ne 0 ]; then 
	echo -e "\n\nERROR : 'make' is not available - please install\n\n"
	echo "hint" >> $haltFile
	# do not stop yet
fi
which gcc > /dev/null
if [ $? -ne 0 ]; then 
	echo -e "\n\nERROR : 'gcc' is not available - please install\n\n"
	echo "hint" >> $haltFile
	# do not stop yet
fi

##### Check for kernel headers
if [ ! -x /lib/modules/${KERN_RELEASE}/build/include/linux/ ]
then
	echo -e "\n\nERROR : The kernel development headers are missing."
	echo "        These are required to build kernel modules, please install."
	echo "hint" >> $haltFile
	# do not stop yet
fi

##### Verify root privileges
USER_ID=`id -u`
if [ ${USER_ID} != 0 ]; then
	echo -e "\n\nERROR : User must be root. \n\n"
	echo -e "try\n $ su -\nor\n $ sudo $0"
	touch $haltFile
	# do not stop yet
fi

# stop now, if any of the above checks failed
if [ -e $haltFile ]; then
	echo -e "\n\nDue to the errors listed above, the installation "
	echo -e "    can not proceed.\n\n"

	if [ -e ./hints.sh ]; then
		grep "hint" $haltFile >/dev/null
		if [ $? -eq 0 ]; then
			source hints.sh
		fi
	fi
	exit
fi

echo === Initial checks complete ====================================

##### Stop opportunity
head -n 10 ../README.txt
read -n1 -p "Continue (y/n)? " agreeYN
# leave the empty echo in place below!
echo 
if [[ "$agreeYN" =~ [nN] ]]; then
	echo -e "\n  Ok. Installation halted."
	touch $haltFile
	exit 99
fi

SRCS1_DIR=/usr/src/modules/zytronic100
SRCS2_DIR=/usr/src/modules/zytronic_mt

echo === Ready to install the Kernel Drivers ========================

echo --- Move the sources to the /usr/src area ----------------------


tar -xvf sources.tgz > /dev/null
mkdir -p ${SRCS1_DIR} > /dev/null
mkdir -p ${SRCS2_DIR} > /dev/null
install -o root -g root -p -m 644 ./drivers100/* ${SRCS1_DIR}
install -o root -g root -p -m 644 ./drivers_mt/* ${SRCS2_DIR}
rm -rf ./drivers*
if [ ! -e ${SRCS1_DIR}/Makefile ]; then
	echo --- ZXY100 Sources missing -----------------------------
	touch $haltFile
	exit 1
fi

if [ ! -e ${SRCS2_DIR}/Makefile ]; then
	echo --- ZXY200 Sources missing -----------------------------
	touch $haltFile
	exit 1
fi

KVERSION=`uname -r`
mkdir -p /lib/modules/${KVERSION}/kernel/drivers/input/touchscreen

pushd ${SRCS1_DIR} > /dev/null
echo --- Make the ZXY100 sources, creating ko files -----------------

make full
# check that the ko files were created
KO_COUNT=`ls -1 *.ko | wc -l`
if [ ${KO_COUNT} -lt 2 ]; then
	echo --- Failed to build the kernel modules -----------------
	popd
	touch $haltFile
	exit 1
fi

echo --- Move the ko files to the correct location ------------------
# and run depmod --quick
echo -e "\n   The depmod command may take some time ...\n"
make install
popd

pushd ${SRCS2_DIR} > /dev/null
echo --- Make the ZXY_MT sources, creating ko files -----------------

make full
# check that the ko files were created
KO_COUNT=`ls -1 *.ko | wc -l`
if [ ${KO_COUNT} -lt 2 ]; then
	echo --- Failed to build the kernel modules -----------------
	popd
	touch $haltFile
	exit 1
fi

echo --- Move the ko files to the correct location ------------------
# and run depmod --quick
echo -e "\n   The depmod command may take some time ...\n"
make install

popd

# check that the files arrived
KO_COUNT=`ls -1 /lib/modules/${KVERSION}/kernel/drivers/input/touchscreen/zxy*.ko | wc -l`
echo Installed $KO_COUNT modules
if [ $KO_COUNT -lt 2 ]; then
	echo --- Failed to find kernel modules - STOP! --------------
	popd > /dev/null
	touch $haltFile
	exit 1
fi


echo --- Insert the modules to the running kernel -------------------
modprobe zxy100_usb
#modprobe zxy100_serial
modprobe zxy_mt_usb

echo --- Verify that the modules loaded
MOD_LIST=`lsmod | grep ^zxy | wc -l`
echo Loaded $MOD_LIST modules to the running kernel
if [ $MOD_LIST -lt 2 ]; then
	echo --- Failed to load modules to kernel -------------------
	popd > /dev/null
	touch $haltFile
	exit 1
fi > /dev/null


echo === Auto-load these modules on system boot =====================

#- Debian, etc.
if [ -e /etc/modules ]; then
	chmod 777 /etc/modules
	echo "zxy100_usb" >> /etc/modules
#	echo "zxy100_serial" >> /etc/modules
	echo "zxy_mt_usb" >> /etc/modules
	chmod 644 /etc/modules
fi

#- Fedora - /etc/sysconfig/module.config - TBC
#- openSUSE - /etc/sysconfig/kernel MODULES_LOADED_ON_BOOT - TBC
#- RedHat - /etc/modules.conf - TBC

echo === Add the init scripts, enable on system startup =============
install -o root -g root -p -m 755 init/zxy_device_rebind /usr/sbin
install -o root -g root -p -m 755 init/zxy_device_rebind_init /etc/init.d
echo === Start the rebinder =========================================

initDone=0
# special reboot handling for some known systems
if [ -e /etc/issue ]; then
	grep Debian /etc/issue > /dev/null
	if  [ $? -eq 0 ]; then
		update-rc.d zxy_device_rebind_init defaults &> /dev/null
		initDone=1
	fi
	
	grep Ubuntu /etc/issue > /dev/null
	if  [ $? -eq 0 ]; then
		update-rc.d zxy_device_rebind_init defaults &> /dev/null
		initDone=1
	fi

	grep Fedora /etc/issue > /dev/null
	if  [ $? -eq 0 ]; then	
		chkconfig --add zxy100_usb_rebind_init &> /dev/null
		/etc/init.d/zxy100_usb_rebind_init start &> /dev/null
		initDone=1
	fi

	grep openSUSE /etc/issue > /dev/null
	if  [ $? -eq 0 ]; then
		/sbin/yast runlevel add service=zxy_device_rebind_init runlevels=2345 &> /dev/null
		initDone=1
	fi
fi

/etc/init.d/zxy_device_rebind_init start > /dev/null

# for unknown systems ... generate the links .. 
if [ $initDone -eq 0 ]; then
	/etc/init.d/zxy_device_rebind_init install &> /dev/null
	echo -e "\nGeneric init Reboot Handling...\n"
fi

## postponed ... 
#install -o root -g root -p -m 755 init/zxy100wd /usr/sbin
#install -o root -g root -p -m 755 init/zxy100wd_init /etc/init.d
#/etc/init.d/zxy100_wd_init install

echo === Zytronic Touchscreen Driver Install================ DONE ===
echo 
